"use strict";
var tinder = require('tinder');
var facebookManagerAsync_1 = require("./facebookManagerAsync");
var jsonFileManagerAsync_1 = require("./jsonFileManagerAsync");
var Promise = require("bluebird");
var utils = require("./utils");
var path = require("path");
var TinderBot = (function () {
    function TinderBot(configuration) {
        this.configuration = configuration;
        this.client = new tinder.TinderClient();
    }
    Object.defineProperty(TinderBot.prototype, "Configuration", {
        get: function () { return this.configuration; },
        enumerable: true,
        configurable: true
    });
    TinderBot.prototype.ProcessHistory = function (client, config) {
        var _this = this;
        if (config.messageOldMatches) {
            console.log('fetching history');
            var getHistory = Promise.promisify(client.getHistory);
            return getHistory().then(function (history) {
                console.log('You have ' + history.matches.length + ' matches');
                if (config.debug) {
                    jsonFileManagerAsync_1.JsonFileManager.SaveJsonAsync(history, 'history' + new Date().getTime() + '.json');
                }
                return Promise.resolve(history.matches).each(function (match) {
                    // Only send message if we have never talked to current match
                    if (!match.messages || match.messages.length === 0) {
                        return Promise.delay(config.sendMessageDelayMs).then(function () {
                            return _this.SendMessageAsync(match.person, client, config);
                        });
                    }
                    return Promise.resolve();
                });
            });
        }
        return Promise.resolve();
    };
    TinderBot.prototype.Run = function () {
        var _this = this;
        return this.AuthorizeTinderAsync(this.configuration).then(function (config) {
            return _this.ProcessHistory(_this.client, config)
                .then(function () {
                return utils.RecurseUntilRejected(function () {
                    return _this.ProcessRecommendationsAsync(_this.client, config);
                });
                // .catch(
                //     (err) => {
                //     }
                //     );
            });
        });
    };
    TinderBot.prototype.PreprocessMessageBody = function (msg, person) {
        msg = utils.ReplaceAllStrings(msg, '$name', person.name);
        msg = utils.ReplaceAllStrings(msg, '$age', utils.Age(person.birth_date).toString());
        return msg;
    };
    TinderBot.prototype.SendMessageAsync = function (person, client, config) {
        var _this = this;
        if (config.messageContent
            && config.messageContent.messages
            && config.messageContent.messages.length) {
            var sendMessage_1 = Promise.promisify(client.sendMessage);
            if (config.messageContent.strategy === 'full') {
                // Each match gets all of the messages in order of appearance
                return Promise.resolve(config.messageContent.messages).each(function (msg) {
                    msg = _this.PreprocessMessageBody(msg, person);
                    return sendMessage_1(person._id, msg);
                });
            }
            else if (config.messageContent.strategy === 'one-random') {
                // Each match gets one message picked at random from the message pool
                var rndIdx = Math.floor(Math.random() * config.messageContent.messages.length);
                var msg = this.PreprocessMessageBody(config.messageContent.messages[rndIdx], person);
                return sendMessage_1(person._id, msg);
            }
        }
        return Promise.resolve();
    };
    TinderBot.prototype.FilterRecommendations = function (recs, config) {
        if (config.filterBioByKeywords && config.filterBioByKeywords.whitelist && config.filterBioByKeywords.whitelist.length) {
            recs = recs.filter(function (res) {
                return config.filterBioByKeywords.whitelist.some(function (keyword) {
                    return res.bio.toUpperCase().indexOf(keyword.toUpperCase()) > -1;
                });
            });
        }
        if (config.filterBioByKeywords && config.filterBioByKeywords.blacklist && config.filterBioByKeywords.blacklist.length) {
            recs = recs.filter(function (res) {
                return config.filterBioByKeywords.blacklist.some(function (keyword) {
                    return res.bio.toUpperCase().indexOf(keyword.toUpperCase()) < 0;
                });
            });
        }
        return recs;
    };
    TinderBot.prototype.ShouldSkipRecommendation = function (rec, config) {
        var skip = false;
        if (config.swipeRightProbability && (Math.random() > config.swipeRightProbability)) {
            skip = true;
        }
        if (config.skipNoBio && !rec.bio) {
            skip = true;
        }
        if (config.skipNoPhoto && !rec.photos && !rec.photos.length) {
            skip = true;
        }
        return skip;
    };
    TinderBot.prototype.ProcessRecommendationsAsync = function (client, config) {
        var _this = this;
        if (!client.isAuthorized()) {
            return Promise.reject(new Error('Tinder not authorized while trying to process recommendations'));
        }
        var getRecommendations = Promise.promisify(client.getRecommendations);
        return getRecommendations(10).then(function (recs) {
            // recs.message seems to be defined only when a limit kicks or something goes wrong on tinder side?
            if (recs && recs.message && recs.message == "recs exhausted") {
                // Ran out of recommendations 
                debugger;
                return Promise.reject(new Error(recs.message));
            }
            else if (recs && recs.message && recs.message == "recs timeout") {
                debugger;
                return Promise.reject(new Error(recs.message));
            }
            else if (recs && recs.message) {
                debugger;
                return Promise.reject(new Error(recs.message));
            }
            else {
                if (config.debug) {
                    jsonFileManagerAsync_1.JsonFileManager.SaveJsonAsync(recs, 'recommendations' + new Date().getTime() + '.json'); //debug only
                }
                console.log('Retrieved ' + recs.results ? recs.results.length : 0 + ' recommendations.');
                var filteredResults = _this.FilterRecommendations(recs.results, config);
                return Promise.resolve(filteredResults).each(function (rec) {
                    var id = rec._id;
                    if (_this.ShouldSkipRecommendation(rec, config)) {
                        console.log('Skipping rec id ' + id);
                        return Promise.reject(new Error('Skipping current rec'));
                    }
                    return Promise.delay(config.swipeRightDelayMs).then(function () {
                        console.log('Liking rec id ' + id);
                        var like = Promise.promisify(client.like);
                        return like(id).then(function (data) {
                            console.log('Likes remaining: ' + (data.likes_remaining ? data.likes_remaining : 0));
                            if (data.matched && config.messageNewMatches) {
                                if (config.debug) {
                                    jsonFileManagerAsync_1.JsonFileManager.SaveJsonAsync(data, 'client.insta.match.on.like' + new Date().getTime() + '.json');
                                }
                                console.log('Like id ' + id + ' matched, sending message.');
                                _this.SendMessageAsync(rec, client, config);
                            }
                            if (!data.likes_remaining) {
                                //probably out of likes, stop looping
                                //in this case data looks like:
                                // likes_remaining:0
                                // match:false
                                // rate_limited_until:1464312608442
                                return Promise.reject(new Error('No more likes available.'));
                            }
                            else {
                                console.log('Liked rec id ' + id);
                            }
                            return data;
                        });
                    });
                });
            }
        });
    };
    TinderBot.prototype.AuthorizeTinderAsync = function (config, recurse) {
        var _this = this;
        if (recurse === void 0) { recurse = true; }
        return this.EnsureFBAuthDataAsync(config).then(function (configWithToken) {
            var authorize = Promise.promisify(_this.client.authorize);
            return authorize(configWithToken.fbUserAccessToken, configWithToken.fbUserId)
                .then(function (res) {
                // Success!
                return configWithToken;
            })
                .catch(function (err) {
                // Failure!
                console.log('Tinder authorization failed, deleting fb access token in config, it might have expired.');
                delete configWithToken.fbUserAccessToken;
                jsonFileManagerAsync_1.JsonFileManager.UpdateTinderConfigAsync(configWithToken, path.resolve('./config.json'));
                if (recurse) {
                    // Try again
                    return _this.AuthorizeTinderAsync(configWithToken, false);
                }
                else {
                    // Give up
                    return Promise.reject(new Error('Tinder authorization failed, unrecoverable authorization failure.')); // or throw exception, same effect
                }
            });
        });
    };
    TinderBot.prototype.EnsureFBAuthDataAsync = function (config) {
        if (!config.fbUserAccessToken || !config.fbUserId) {
            if (!config.user || !config.password) {
                return Promise.reject(new Error('Config section contains too litle information.'));
            }
            return facebookManagerAsync_1.FacebookManager.getAccessTokenAndIdAsync(config.user, config.password).then(function (results) {
                config.fbUserAccessToken = results[0];
                config.fbUserId = results[1];
                var foo = jsonFileManagerAsync_1.JsonFileManager.UpdateTinderConfigAsync(config, path.resolve('./config.json'))
                    .thenReturn(config);
                return foo;
            });
        }
        else {
            return Promise.resolve(config);
        }
    };
    return TinderBot;
}());
exports.TinderBot = TinderBot;
//# sourceMappingURL=tinderBotAsync.js.map